"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _cleanWebpackPlugin = require("clean-webpack-plugin");

var _miniCssExtractPlugin = _interopRequireDefault(require("mini-css-extract-plugin"));

var _webpackPwaManifest = _interopRequireDefault(require("webpack-pwa-manifest"));

var _webpackBundleAnalyzer = require("webpack-bundle-analyzer");

var _terserWebpackPlugin = _interopRequireDefault(require("terser-webpack-plugin"));

var _ = require("..");

require("../bootstrap");

var _getEntries = _interopRequireDefault(require("../lib/get-entries"));

var _getExistConfigPath = _interopRequireDefault(require("../lib/get-exist-config-path"));

/**
 * webpack编译环境配置文件
 * @author Joe Zhong <zhong.zhi@163.com>
 * @module config/webpack.build.babel
 */
const {
  NODE_ENV,
  CDN_ROOT = '/'
} = process.env;
const context = (0, _.getContext)();
const appConfig = (0, _.pRequire)('config/packing');
const {
  assetExtensions,
  commonChunks,
  runtimeChunk: {
    enabled: runtimeChunkEnabled,
    name: runtimeChunkName
  },
  longTermCaching: {
    enabled: longTermCachingEnabled,
    options: {
      delimiter,
      fileHashLength
    }
  },
  cssLoader,
  template: {
    enabled: templateEnabled,
    options: {
      injectManifest
    }
  },
  eslint: {
    enabled: eslintEnabled,
    options: eslintOptions
  },
  bundleAnalyzer: {
    enabled: bundleAnalyzerEnabled,
    options: bundleAnalyzerOptions
  },
  minimize: {
    enabled: minimizeEnabled,
    options: minimizeOptions
  },
  path: {
    src: {
      root: srcRoot
    },
    dist: {
      root: distRoot,
      js,
      css
    },
    entries
  }
} = appConfig;

const getHashPattern = type => {
  let hashPattern = '';

  if (longTermCachingEnabled) {
    hashPattern = `${delimiter}[${type}:${fileHashLength}]`;
  }

  return hashPattern;
};
/**
 * 生成webpack配置文件
 * @param {object} program 程序进程，可以获取启动参数
 * @param {object} options 特征配置项
 * @return {object}
 */


const webpackConfig = () => {
  const outputPath = _path.default.resolve(context, distRoot);

  const chunkhash = getHashPattern('chunkhash');
  const contenthash = getHashPattern('contenthash');
  const hash = getHashPattern('hash');
  const entry = (0, _getEntries.default)(entries);
  const output = {
    chunkFilename: `${js}/[name]${chunkhash}.js`,
    filename: `${js}/[name]${chunkhash}.js`,
    // prd环境静态文件输出地址
    path: outputPath,
    // dev环境下数据流访问地址
    publicPath: CDN_ROOT
  };
  const cssLoaderOptions = cssLoader;
  const postcssLoaderOptions = {
    // 兼容 eslint-loader@4.x
    postcssOptions: {
      config: (0, _getExistConfigPath.default)('postcss', context, __dirname)
    }
  };
  const jsLoaders = [{
    loader: 'babel-loader'
  }];

  if (eslintEnabled) {
    jsLoaders.push({
      loader: 'eslint-loader',
      options: eslintOptions
    });
  }

  const module = {
    rules: [{
      test: /\.(t|j)sx?$/i,
      exclude: /node_modules/,
      use: jsLoaders
    }, {
      test: /\.css$/i,
      use: [{
        loader: _miniCssExtractPlugin.default.loader
      }, {
        loader: 'css-loader',
        options: cssLoaderOptions
      }, {
        loader: 'postcss-loader',
        options: postcssLoaderOptions
      }]
    }, {
      test: /\.(scss|sass)$/i,
      use: [{
        loader: _miniCssExtractPlugin.default.loader
      }, {
        loader: 'css-loader',
        options: cssLoaderOptions
      }, {
        loader: 'postcss-loader',
        options: postcssLoaderOptions
      }, {
        loader: 'sass-loader'
      }]
    }, {
      test: /\.less$/i,
      use: [{
        loader: _miniCssExtractPlugin.default.loader
      }, {
        loader: 'css-loader',
        options: cssLoaderOptions
      }, {
        loader: 'postcss-loader',
        options: postcssLoaderOptions
      }, {
        loader: 'less-loader'
      }]
    }, {
      test: new RegExp(`.(${assetExtensions.join('|')})$`, 'i'),
      loader: 'file-loader',
      options: {
        // context 参数会影响静态文件打包输出的路径
        name: `[path][name]${hash}.[ext]`
      }
    }]
  };
  const resolve = {
    modules: [srcRoot, 'node_modules'],
    extensions: ['.tsx', '.jsx', '.ts', '.js']
  };
  const plugins = [new _cleanWebpackPlugin.CleanWebpackPlugin(), new _miniCssExtractPlugin.default({
    // Options similar to the same options in webpackOptions.output
    // both options are optional
    filename: `${css}/[name]${contenthash}.css`,
    // filename: '[name].css',
    chunkFilename: `${css}/[id]${contenthash}.css`
  })];

  if (templateEnabled) {
    plugins.push(new _.plugin(appConfig));
  }

  if (injectManifest) {
    plugins.push(new _webpackPwaManifest.default({ ...(0, _.requireDefault)(_path.default.resolve(context, 'config/webpack.manifest')),
      ...{
        filename: `[name]${hash}[ext]`
      }
    }));
  } // 该插件用的还是旧插件机制


  if (bundleAnalyzerEnabled) {
    plugins.push(new _webpackBundleAnalyzer.BundleAnalyzerPlugin(bundleAnalyzerOptions));
  }

  const optimization = {
    minimize: minimizeEnabled
  }; // const optimization = {};

  if (minimizeEnabled) {
    optimization.minimizer = [new _terserWebpackPlugin.default(minimizeOptions)];
  }

  if (runtimeChunkEnabled) {
    optimization.runtimeChunk = {
      name: runtimeChunkName
    };
  }

  if (commonChunks && Object.keys(commonChunks).length > 0) {
    const cacheGroups = {};
    Object.keys(commonChunks).forEach(chunkName => {
      const pattern = commonChunks[chunkName].join('|');
      cacheGroups[chunkName] = {
        test: new RegExp(`(${pattern})`),
        // /(ccc|sub\/bbb|\.\/d)/,
        chunks: 'initial',
        name: chunkName,
        // 优先级
        priority: 10,
        // 忽略命中次数，只要命中且使用过一次就打入 vendor
        enforce: true
      };
    });
    optimization.splitChunks = {
      cacheGroups
    };
  }

  return {
    mode: NODE_ENV !== 'production' ? 'development' : 'production',
    context,
    entry,
    output,
    optimization,
    module,
    resolve,
    plugins
  };
};

var _default = program => webpackConfig(program);

exports.default = _default;
module.exports = exports.default;