"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = require("fs");

var _path = require("path");

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _loaderUtils = _interopRequireDefault(require("loader-utils"));

var _packingGlob = _interopRequireDefault(require("packing-glob"));

var _ = require("..");

var _getEntries = _interopRequireDefault(require("./get-entries"));

var _getEntryFromList = _interopRequireDefault(require("./get-entry-from-list"));

/**
 * 思路：
 *   1. 在 done 时根据 entries 信息生成网页并存储到 this.pages
 *      如果 autoGeneration=false 就读取存在的模版文件并存储到 this.pages
 *   2. 匹配出所有静态资源，生成 matches
 *   3. 根据 matches 信息拷贝文件并使用 hash 重命名文件名
 *   4. 把 pages 中的文件替换为新的文件路径
 *   5. 根据 assets 中输出的文件，动态向 pages 中插入脚本和样式
 *   6. 输出 html 文件
 */

/**
webpack plugin hooks 执行顺序：
--environment--
--afterEnvironment--
--entryOption--
--afterPlugins--
--afterResolvers--
--beforeRun--
--run--
--normalModuleFactory--
--contextModuleFactory--
--beforeCompile--
--compile--
--thisCompilation--
--compilation--
--make--
--afterCompile--
--shouldEmit--
--emit--
--afterEmit--
--done--
 */
class PackingTemplatePlugin {
  constructor(appConfig) {
    this.context = (0, _.getContext)();
    this.appConfig = appConfig;
    this.pages = {};
  }

  apply(compiler) {
    // webpack v4
    if (compiler.hooks) {
      compiler.hooks.done.tap(this.constructor.name, stats => {
        this.done(compiler, stats);
      });
    } else {
      compiler.plugin('done', stats => {
        this.done(compiler, stats);
      });
    }
  }

  done(compiler, stats) {
    const {
      template: {
        options: {
          engine,
          autoGeneration
        }
      }
    } = this.appConfig;
    const statsJson = stats.compilation.getStats().toJson({
      all: false,
      entrypoints: true
    }); // 记录所有页面入口和入口对应的 assets

    this.entrypoints = statsJson.entrypoints;

    if (autoGeneration) {
      this.generatePages();
    } else {
      this.readPages();
    }

    this.getAssetsMap(compiler);
    this.output(compiler);

    if (engine === 'pug') {
      this.copyAndReplaceLayout(compiler);
    }
  }

  generatePages() {
    const {
      path: {
        entries
      },
      commonChunks,
      template: {
        options: templateOptions
      }
    } = this.appConfig; // 该 entries 信息包含 commonChunks 配置

    const entryPoints = (0, _getEntries.default)(entries);
    Object.keys(entryPoints) // 排除 commonChunks 入口
    .filter(entry => Object.keys(commonChunks).indexOf(entry) < 0).forEach(chunkName => {
      let settings = {};
      const entryFile = (0, _getEntryFromList.default)(chunkName, entryPoints[chunkName]);
      const settingsFile = (0, _path.resolve)(this.context, entryFile.replace(/(\.(t|j)s)/, '.settings$1'));

      if ((0, _fs.existsSync)(settingsFile)) {
        settings = (0, _.requireDefault)(settingsFile);
      }

      const args = { ...templateOptions,
        ...settings
      };
      const {
        title,
        master,
        favicon,
        keywords,
        description,
        ...templateData
      } = args;
      let html = '';
      const parent = (0, _path.isAbsolute)(master) ? master : (0, _path.resolve)(this.context, master);

      if ((0, _fs.existsSync)(parent)) {
        const templateString = (0, _fs.readFileSync)(parent, {
          encoding: 'utf-8'
        });
        html = templateString;
      } else {
        throw new Error(`\n找不到母模版：${parent}\n`);
      }

      html = this.injectTitle(html, title);
      html = this.injectMeta(html, favicon, keywords, description); // 替换格式为 __var__ 用户自定义变量

      html = html.replace(/__(\w+)__/gm, (re, $1) => templateData[$1] || '');
      this.pages[chunkName] = {
        args,
        html
      };
    });
  }
  /**
   * 读取已有的模版文件
   */


  readPages() {
    const {
      path: {
        src: {
          root: src,
          templates
        }
      },
      template: {
        options: {
          extension
        }
      }
    } = this.appConfig;
    const tempagePages = templates.pages || templates;
    const templateRoot = (0, _path.resolve)((0, _.getContext)(), src, tempagePages);
    (0, _packingGlob.default)(`**/*${extension}`, {
      cwd: templateRoot
    }).forEach(file => {
      const html = (0, _fs.readFileSync)((0, _path.resolve)(templateRoot, file), {
        encoding: 'utf-8'
      });
      const args = this.appConfig.template.options;
      this.pages[file.replace(extension, '')] = {
        args,
        html
      };
    });
  }

  getAssetsMap(compiler) {
    const {
      engine,
      path: outputPath
    } = this.appConfig.template.options;
    Object.keys(this.pages).forEach(chunkName => {
      const matches = [];
      const {
        args,
        html
      } = this.pages[chunkName];
      args.attrs.forEach(a => {
        const {
          tag,
          attribute
        } = this.parseAttribute(a);
        const reg = engine === 'pug' ? new RegExp(`${tag}(?:\\(.*\\s+|\\()(?:${attribute})\\s*=\\s*["']([^"']+)`, 'g') : new RegExp(`${tag}.*\\s+(?:${attribute})\\s*=\\s*["']([^"']+)`, 'g');
        let result;
        console.log('---reg:', reg); // eslint-disable-next-line no-cond-assign

        while (result = reg.exec(html)) {
          const value = result[1];

          if (!/^(https{0,1}:){0,1}\/\//.test(value)) {
            const head = result[0].replace(value, ''); // => src="

            const file = (0, _path.join)(this.context, value);

            if ((0, _fs.existsSync)(file) && (0, _fs.statSync)(file).isFile()) {
              const content = (0, _fs.readFileSync)(file);
              const {
                name,
                ext,
                dir
              } = (0, _path.parse)(value);
              const hash = this.getHashDigest(content);
              const newValue = outputPath.replace('[path]', dir ? `${dir}/` : '').replace('[name]', name).replace('[ext]', ext.replace('.', '')).replace(/\[hash(:{0,1}\d{0,2})\]/, pattern => {
                let hashLength = 32;

                if (pattern) {
                  const hashLengthMatches = pattern.match(/\[hash:(\d{1,2})\]/);

                  if (hashLengthMatches && hashLengthMatches[1]) {
                    [, hashLength] = hashLengthMatches; // eslint disable line
                  }
                }

                return hash.substr(0, hashLength);
              });
              const dist = (0, _path.join)(compiler.options.output.path, newValue);

              _mkdirp.default.sync((0, _path.dirname)(dist));

              if (!(0, _fs.existsSync)(dist)) {
                (0, _fs.writeFileSync)(dist, content);
              }

              matches.push({
                start: result.index + head.length,
                length: value.length,
                value,
                newValue
              });
            }
          }
        }
      });
      matches.sort((a, b) => {
        if (a.start > b.start) {
          return 1;
        } else if (a.start < b.start) {
          return -1;
        } else {
          return 0;
        }
      });
      this.pages[chunkName].matches = matches;
    });
  }

  output(compiler) {
    const {
      path: {
        dist: {
          root: dist,
          templates
        }
      },
      template: {
        options: {
          extension,
          injectManifest
        }
      }
    } = this.appConfig;
    const templatePages = templates.pages || templates;
    let {
      publicPath
    } = compiler.options.output;

    if (!publicPath.endsWith('/')) {
      publicPath = `${publicPath}/`;
    }

    Object.keys(this.pages).forEach(chunkName => {
      const {
        matches,
        args: {
          inject
        }
      } = this.pages[chunkName];
      let {
        html
      } = this.pages[chunkName];
      html = html.split(''); // 替换模版中的 src="x.png" -> src="x_xxxxxxxx.png"

      matches.reverse().forEach(link => {
        const url = publicPath + (link.newValue.startsWith('/') ? link.newValue.substring(1, link.newValue.length) : link.newValue);
        html.splice(link.start, link.length, url);
      });
      html = html.join(''); // 当前页面使用到的所有 asets

      const {
        assets
      } = this.entrypoints[chunkName];

      if (inject && assets) {
        html = this.injectStyles(html, chunkName, assets, publicPath);
        html = this.injectScripts(html, chunkName, assets, publicPath);
      }

      if (injectManifest) {
        html = this.injectManifest(html, this.getManifestFileName(compiler), publicPath);
      }

      const filename = (0, _path.resolve)(this.context, dist, templatePages, `${chunkName + extension}`);

      _mkdirp.default.sync((0, _path.dirname)(filename));

      (0, _fs.writeFileSync)(filename, html);
    });
  }

  copyAndReplaceLayout(compiler) {
    const {
      path: {
        src: {
          root: src,
          templates: {
            layout: srcLayout
          }
        },
        dist: {
          root: distRoot,
          templates: {
            layout: distLayout
          }
        }
      },
      template: {
        options: templateOptions
      }
    } = this.appConfig;
    const {
      attrs
    } = templateOptions;
    let {
      publicPath
    } = compiler.options.output;

    if (!publicPath.endsWith('/')) {
      publicPath = `${publicPath}/`;
    }

    const layouts = (0, _packingGlob.default)('**/*.pug', {
      cwd: (0, _path.resolve)(this.context, src, srcLayout)
    });
    layouts.forEach(layout => {
      const absPath = (0, _path.resolve)(this.context, src, srcLayout, layout);
      let html = (0, _fs.readFileSync)(absPath, {
        encoding: 'utf-8'
      });
      const matches = [];
      attrs.forEach(a => {
        const {
          tag,
          attribute
        } = this.parseAttribute(a);
        const reg = new RegExp(`${tag}(?:\\(.*\\s+|\\()(?:${attribute})\\s*=\\s*["']([^"']+)`, 'g');
        let result; // eslint-disable-next-line no-cond-assign

        while (result = reg.exec(html)) {
          const value = result[1];

          if (!/^(https{0,1}:){0,1}\/\//.test(value)) {
            const head = result[0].replace(value, ''); // => src="

            const file = (0, _path.resolve)(this.context, value);

            if ((0, _fs.existsSync)(file) && (0, _fs.statSync)(file).isFile()) {
              const content = (0, _fs.readFileSync)(file);
              const {
                name,
                ext,
                dir
              } = (0, _path.parse)(value);
              const hash = this.getHashDigest(content);
              const newValue = templateOptions.path.replace('[path]', dir ? `${dir}/` : '').replace('[name]', name).replace('[ext]', ext.replace('.', '')).replace(/\[hash(:{0,1}\d{0,2})\]/, pattern => {
                let hashLength = 32;

                if (pattern) {
                  const hashLengthMatches = pattern.match(/\[hash:(\d{1,2})\]/);

                  if (hashLengthMatches && hashLengthMatches[1]) {
                    [, hashLength] = hashLengthMatches;
                  }
                }

                return hash.substr(0, hashLength);
              });
              const dist = (0, _path.resolve)(compiler.options.output.path, newValue);

              _mkdirp.default.sync((0, _path.dirname)(dist));

              if (!(0, _fs.existsSync)(dist)) {
                (0, _fs.writeFileSync)(dist, content);
              }

              matches.push({
                start: result.index + head.length,
                length: value.length,
                value,
                newValue
              });
            }
          }
        }
      });
      matches.sort((a, b) => {
        if (a.start > b.start) {
          return 1;
        } else if (a.start < b.start) {
          return -1;
        } else {
          return 0;
        }
      }); // 输出 layout

      html = html.split('');
      matches.reverse().forEach(link => {
        const url = publicPath + link.newValue;
        html.splice(link.start, link.length, url);
      });
      const filename = (0, _path.resolve)(this.context, distRoot, distLayout, layout);

      _mkdirp.default.sync((0, _path.dirname)(filename));

      (0, _fs.writeFileSync)(filename, html.join(''));
    });
  }

  getHashDigest(content) {
    return _loaderUtils.default.getHashDigest(content);
  }

  parseAttribute(attribute) {
    const arr = attribute.split(':');

    if (arr.length !== 2) {
      throw new Error(`[error] PackingTemplatePlugin: attributes 参数格式错误：${attribute}`);
    }

    return {
      tag: arr[0] === '*' ? '' : arr[0],
      attribute: arr[1].replace('-', '\\-')
    };
  }

  getManifestFileName(compiler) {
    let pwaAssets;
    compiler.options.plugins.filter(p => p.constructor.name === 'WebpackPwaManifest').forEach(p => {
      pwaAssets = p.assets;
    });

    if (pwaAssets) {
      return pwaAssets.filter(asset => /manifest\w*.json/.test(asset.output))[0].output;
    }

    return '';
  }

  injectTitle(html, title) {
    const {
      template: {
        options: {
          engine
        }
      }
    } = this.appConfig;

    if (title) {
      // 为 SEO 准备的页面 meta 信息
      if (engine === 'pug') {
        html = `${html}\nblock title\n  title ${title}\n`;
      } else {
        html = html.replace(/__title__/g, title);
      }
    }

    return html;
  }

  injectMeta(html, favicon, keywords, description) {
    const {
      template: {
        options: {
          engine
        }
      }
    } = this.appConfig; // 为 SEO 准备的页面 meta 信息

    if (engine === 'pug') {
      const metaTags = [];

      if (keywords) {
        metaTags.push(`  meta(name="keywords" content="${keywords}")`);
      }

      if (description) {
        metaTags.push(`  meta(name="description" content="${description}")`);
      }

      if (favicon) {
        metaTags.push(`  link(rel="icon" type="image/png" href="${favicon}")`);
      }

      const metaHtml = metaTags.join('\n');

      if (metaHtml) {
        html = `${html}\nblock append meta\n${metaHtml}\n`;
      }
    } else {
      const metaTags = [];

      if (keywords) {
        metaTags.push(`  <meta name="keywords" content="${keywords}">`);
      }

      if (description) {
        metaTags.push(`  <meta name="description" content="${description}">`);
      }

      if (favicon) {
        metaTags.push(`  <link rel="icon" type="image/png" href="${favicon}">`);
      }

      const metaHtml = metaTags.join('\n');

      if (metaHtml) {
        html = html.replace('</head>', `${metaHtml}\n  </head>`);
      }
    }

    return html;
  }

  injectManifest(html, filename, publicPath) {
    const {
      template: {
        options: {
          engine
        }
      }
    } = this.appConfig; // 为 SEO 准备的页面 meta 信息

    if (engine === 'pug') {
      return `${html}\nblock append meta\n  link(rel="manifest" href="${publicPath}${filename}")\n`;
    }

    return html.replace('</head>', `  <link rel="manifest" href="${publicPath}${filename}">\n  </head>`);
  }

  injectStyles(html, chunkName, assets, publicPath) {
    const {
      template: {
        options: {
          engine
        }
      }
    } = this.appConfig;
    const styles = assets.filter(({
      name
    }) => name.endsWith('.css'));

    if (styles.length > 0) {
      let styleHtml;

      if (engine === 'pug') {
        styleHtml = `block append style\n${styles.map(({
          name
        }) => `  link(href="${publicPath + name}" rel="stylesheet")`).join('\n')}`;
        html = `${html}\n${styleHtml}\n`;
      } else {
        styleHtml = styles.map(({
          name
        }) => `  <link href="${publicPath + name}" rel="stylesheet">`).join('\n');
        html = html.replace('</head>', `${styleHtml}\n  </head>`);
      }
    }

    return html;
  }

  injectScripts(html, chunkName, assets, publicPath) {
    const {
      template: {
        options: {
          engine,
          scriptInjectPosition
        }
      }
    } = this.appConfig;
    const scripts = assets.filter(({
      name
    }) => name.endsWith('.js'));

    if (engine === 'pug') {
      const scriptPug = scripts.map(({
        name
      }) => `  script(src="${publicPath + name}")`).join('\n');
      html = `${html}\nblock append script\n${scriptPug}\n`;
    } else {
      const scriptHtml = scripts.map(({
        name
      }) => `  <script src="${publicPath + name}"></script>`).join('\n');
      html = html.replace(`</${scriptInjectPosition}>`, `${scriptHtml}\n  </${scriptInjectPosition}>`);
    }

    return html;
  }

}

exports.default = PackingTemplatePlugin;
module.exports = exports.default;